% --- Source file: man/define.NB.geno.Rd ---
\name{define.NB.geno}
\alias{define.NB.geno}
\title{
Define a subject grouping and similarity matrix for the groups}
\description{
Define a subject grouping and similarity matrix for the groups using genotype data
}
\usage{
define.NB.geno(geno.mat, num.neighbor=4, method.tie="min")
}
\arguments{
  \item{geno.mat}{Matrix of genotype data. Genotype should be coded as 0, 1, or 2 for the number of copies of
                  the minor allele. The dimension of this matrix must be the number of subjects by the 
                  number of SNPs.}
  \item{num.neighbor}{The number of neighbors for the similarity matrix.}
  \item{method.tie}{One of "average", "first", "random", "max", "min".}
}
\value{
  A list containing the subject grouping (\code{grp.subj}) and similarity matrix (\code{NB.mat}).
  The order of \code{grp.subj} is the same as the order of the subjects in \code{geno.mat}.
  The groups will be coded as 1, 2, ..., Ngroups. The dimension of \code{NB.mat} will be
  Ngroups by Ngroups, where the ith row is for group number i.
}
\details{
  This function can be called to obtain a vector of groups for the subjects and a similarity matrix
  for the groups that can be used as input for the \code{\link{badge}} function.
  Subject with a similar genotype structure will be put into the same group.

}
\seealso{\code{\link{badge}}}
\author{Kai Yu}
\examples{

  # Create a matrix of 0, 1, and 2
  set.seed(123)
  nsub <- 100
  nsnp <- 5
  mat  <- rbinom(nsub*nsnp, 2, 0.45)
  dim(mat) <- c(nsub, nsnp)

  ret <- define.NB.geno(mat)
  table(ret$grp.subj)
  ret$NB.mat[1:5, 1:5]

}