% --- Source file: man/run_SAMC.Rd ---
\name{run_SAMC}
\alias{run_SAMC}
\title{Psi Grid Points}
\description{Creates a file of grid points for psi}
\usage{
run_SAMC(data, cc.var, exposure.var, group.var, out.dir, op=NULL)}

\arguments{
  \item{data}{Data frame containing the disease status, exposure variable, group variable and possibly covariates.}
  \item{cc.var}{Variable name for the disease status. This variable should be coded as 0 for no disease 
               and 1 for disease.}
  \item{exposure.var}{Variable name for the exposure. This variable should be numerically coded.}
  \item{group.var}{Variable name for the groups. This variable should be coded as integers from 1 to the number of groups.}
  \item{out.dir}{Directory where the output files will be written.}
  \item{op}{List of options. See \code{details} for all possible options.}
}
\value{
  The returned value is NULL. All output files are written to \code{out.dir}.
 The output file will be called paste("psi_grid", out.string, "_", k_max, ".txt", sep="")
}
\details{
  This function can be called prior to calling the \code{\link{badge}} function with option \code{method_psi} = 2.
  Then input data should only contain finite values for the variables to be used in the analysis.

 \bold{Options list:} \cr

 Below are the names for the options list \code{op}. All names have default values
         if they are not specified. See \code{\link{badge}} for other options.
  \itemize{
  \item \code{delete} 0 or 1 to delete the temporary files written to \code{out.dir}. The default is 1.
  \item \code{num_iter} Number of iterations. The default is 10000000. 
  \item \code{gain_factor_t0} The default is 50000. 
  \item \code{psi_vec} The default is 0.1*c(0:12). 

  } % END: itemize

}
\seealso{\code{\link{badge}}}
\author{Kai Yu}
\examples{

  set.seed(123)
  n    <- 100
  cc   <- rbinom(n, 1, 0.5)
  x    <- rbinom(n, 1, 0.5)
  grp  <- sample(1:20, n, replace=TRUE) 
  data <- data.frame(cc, x, grp)
  dir  <- "K:/bayesian/R_package/temp/"

  # Not run  
  #run_SAMC(data, "cc", "x", "grp", dir)
  
}