#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "./header.h"

FILE * open_file(datafile, mode)
const char *datafile, *mode;
{
  FILE *fpt;

  fpt = fopen(datafile, mode);
  if (fpt == 0) {
    printf("\n Error opening file: %s \n", datafile);
    exit(0);
  }

  return(fpt);
}



/* Function to return the median of a double vector */
double median(vec, n)
double *vec;
int n;
{
  int temp[n];
  double ret;

  quicksort(0, n-1, vec, temp);
  if (n % 2) {
    /* n is odd */
    ret = vec[(n+1)/2 - 1];
  } else {
    /* n is even */
    ret = (vec[n/2 - 1] + vec[n/2])/2.0;
  }

  return(ret);

} /* END: median */

/* Function to return the z matrix */
void get_Z(M1, M2, everyN, Niter, Ngrp, z_file, Dmat)
 int *M1, *M2, *everyN, *Niter, *Ngrp;
 int *Dmat; /* matrix initialized to 0*/
 char **z_file;
{
  int zint, i, j, eofFlag,  MM1, MM2, everyNN, *ptri, ii, jj, *Z, ngrp, Nii;
  FILE *fpt;

  MM1     = *M1;
  MM2     = *M2;
  ngrp    = *Ngrp;
  everyNN = *everyN;
  
  /* Z vector */
  Z = (int *) malloc(ngrp*sizeof(int));
  CHECK_MEM(Z);

  /* Open the Z file */
  fpt = open_file(*z_file, "r");

  eofFlag = 0;
  Nii     = 0;

  for(i=1; i<=*Niter; i++) {

    if (feof(fpt)) break;
    if (i > MM2) break;    

    /* Read in a row from the Z file */
    for (j=0, ptri=Z; j<ngrp; j++, ptri++) {
      ii = fscanf(fpt, "%d", &zint);
      if (ii == EOF) {
        eofFlag = 1;
        break;
      }
      *ptri = zint;
    }
    if (eofFlag) break;
    if (i < MM1) continue;
    
    /* Use every NN observations */
    Nii++;
    if (Nii == everyNN) {
      /* Reset Nii */
      Nii = 0;
    } else {
      /* Skip this observation */
      continue;
    }

    /* Update D */
    for (ii=0; ii<ngrp; ii++) {
      for (jj=ii; jj<ngrp; jj++) {
        if (Z[ii] == Z[jj]) Dmat[ngrp*ii + jj]++;
      }
    }
  }
  fclose(fpt);
  free(Z);

  /* Lower triangle of D */
  for (ii=1; ii<ngrp; ii++) {
    for (jj=0; jj<ii; jj++) Dmat[ii*ngrp + jj] = Dmat[jj*ngrp + ii];
  }
  
}


/* Function to return the mean of alpha and beta*/
void get_alpha_beta(M1, M2, everyN, Nsub, Niter, Nclus, Ngrp, z_file, grp_id, alpha_file, beta_file, alpha_mean, beta_mean)
 int *M1, *M2, *everyN, *Nsub, *Niter, *Nclus, *Ngrp, *grp_id;
 double *alpha_mean, *beta_mean; /* length Nsub */
 char **z_file, **alpha_file, **beta_file;
{
  int zint, i, j, k, eofFlag, N, MM1, MM2, *ptri, *Z, ngrp, ii, nclus, count, everyNN, Nii;
  FILE *fpt, *fpta, *fptb;
  double alpha[*Nclus], beta[*Nclus], *ptrd, *ptra, *ptrb, amean[*Ngrp], bmean[*Ngrp];
  float x;

  N       = *Nsub;
  MM1     = *M1;
  MM2     = *M2;
  nclus   = *Nclus;
  ngrp    = *Ngrp;
  everyNN = *everyN;
  
  /* Z vector */
  Z = (int *) malloc(ngrp*sizeof(int));
  CHECK_MEM(Z);

  for (i=0, ptra=amean, ptrb=bmean; i<ngrp; i++, ptra++, ptrb++) {
    *ptra = 0.0;
    *ptrb = 0.0;
  }

  /* Open the Z file */
  fpt  = open_file(*z_file, "r");
  fpta = open_file(*alpha_file, "r");
  fptb = open_file(*beta_file, "r");

  eofFlag = 0;
  count = 0;
  Nii = 0;

  for(i=1; i<=*Niter; i++) {

    if (feof(fpt)) break;
    if (i > MM2) break;    

    /* Read in a row from the Z file */
    for (j=0, ptri=Z; j<ngrp; j++, ptri++) {
      ii = fscanf(fpt, "%d", &zint);
      if (ii == EOF) {
        eofFlag = 1;
        break;
      }
      *ptri = zint;
    }

    /* Read in a row from alpha and beta files */
    for (j=0, ptrd=alpha; j<nclus; j++, ptrd++) {
      ii = fscanf(fpta, "%g", &x);
      *ptrd = x;
    }
    for (j=0, ptrd=beta; j<nclus; j++, ptrd++) {
      ii = fscanf(fptb, "%g", &x);
      *ptrd = x;
    }

    if (eofFlag) break;
    if (i < MM1) continue;

    /* Use every NN observations */
    Nii++;
    if (Nii == everyNN) {
      /* Reset Nii */
      Nii = 0;
    } else {
      /* Skip this observation */
      continue;
    }

    /* Update alpha and beta */
    for (j=0, ptri=Z, ptra=amean, ptrb=bmean; j<ngrp; j++, ptri++, ptra++, ptrb++) {
      k = *ptri;
      *ptra += alpha[k];
      *ptrb += beta[k];
    }
    count++;
  }
  fclose(fpt);
  fclose(fpta);
  fclose(fptb);
  free(Z);

  for (j=0, ptra=amean, ptrb=bmean; j<ngrp; j++, ptra++, ptrb++) {
    *ptra /= count;
    *ptrb /= count;
  }

  /* Get the result for each subject */
  for (i=0, ptra=alpha_mean, ptrb=beta_mean, ptri=grp_id; i<*Nsub; i++, ptra++, ptrb++, ptri++) {
    j = *ptri;  /* j is the group id for subject i*/
    *ptra = amean[j];
    *ptrb = bmean[j];
  }

}

/* Function to return all alpha and beta*/
void median_alpha_beta(M1, M2, everyN, Nsub, Niter, Nclus, Ngrp, z_file, grp_id, alpha_file, beta_file, alpha_med, beta_med)
 int *M1, *M2, *everyN, *Nsub, *Niter, *Nclus, *Ngrp, *grp_id;
 double *alpha_med, *beta_med; /* length Nsub */
 char **z_file, **alpha_file, **beta_file;
{
  int zint, i, j, k, eofFlag, N, MM1, MM2, *ptri, *Z, ngrp, ii, nclus, count, everyNN, Nii, niter, nkeep;
  FILE *fpt, *fpta, *fptb;
  double *alpha, *beta, *ptrd, *ptra, *ptrb, **A, **B;
  float x;

  N       = *Nsub;
  MM1     = *M1;
  MM2     = *M2;
  nclus   = *Nclus;
  ngrp    = *Ngrp;
  everyNN = *everyN;
  niter   = *Niter;
  
  /* Z vector */
  Z = (int *) malloc(ngrp*sizeof(int));
  CHECK_MEM(Z);

  alpha = malloc(nclus*sizeof(double));
  CHECK_MEM(alpha);
  beta = malloc(nclus*sizeof(double));
  CHECK_MEM(beta);

  /* Get the number of iterations to store */
  nkeep = ceil((MM2 - MM1 + 1)/everyNN);

  /* Allocate matrices of size ngrp x nkeep for alpha and beta */
  A = dblMatrix_alloc(ngrp, nkeep);
  B = dblMatrix_alloc(ngrp, nkeep);

  /* Open the Z file */
  fpt  = open_file(*z_file, "r");
  fpta = open_file(*alpha_file, "r");
  fptb = open_file(*beta_file, "r");

  eofFlag = 0;
  count = -1;
  Nii = 0;

  for(i=1; i<=niter; i++) {

    if (feof(fpt)) break;
    if (i > MM2) break;    

    /* Read in a row from the Z file */
    for (j=0, ptri=Z; j<ngrp; j++, ptri++) {
      ii = fscanf(fpt, "%d", &zint);
      if (ii == EOF) {
        eofFlag = 1;
        break;
      }
      *ptri = zint;
    }

    /* Read in a row from alpha and beta files */
    for (j=0, ptrd=alpha; j<nclus; j++, ptrd++) {
      ii = fscanf(fpta, "%g", &x);
      *ptrd = x;
    }
    for (j=0, ptrd=beta; j<nclus; j++, ptrd++) {
      ii = fscanf(fptb, "%g", &x);
      *ptrd = x;
    }

    if (eofFlag) break;
    if (i < MM1) continue;

    /* Use every NN observations */
    Nii++;
    if (Nii == everyNN) {
      /* Reset Nii */
      Nii = 0;
    } else {
      /* Skip this observation */
      continue;
    }

    /* Store these values */
    count++;
    for (j=0, ptri=Z; j<ngrp; j++, ptri++) {
      k = *ptri;
      A[j][count] = alpha[k];
      B[j][count] = beta[k];
    }

  }
  fclose(fpt);
  fclose(fpta);
  fclose(fptb);
  free(Z);
  free(alpha);
  free(beta);

  /* alpha and beta will hold the medians for each group */
  alpha = malloc(ngrp*sizeof(double));
  CHECK_MEM(alpha);
  beta = malloc(ngrp*sizeof(double));
  CHECK_MEM(beta);

  /* Get the median for each group */
  count = count + 1;  /* Add 1 to get the number of iterations included */
  for (i=0; i<ngrp; i++) {
    alpha[i] = median(A[i], count);
    beta[i]  = median(B[i], count);
  }  

  dblMatrix_free(A, ngrp);
  dblMatrix_free(B, ngrp);

  /* Get the result for each subject */
  for (i=0, ptra=alpha_med, ptrb=beta_med, ptri=grp_id; i<N; i++, ptra++, ptrb++, ptri++) {
    j = *ptri;  /* j is the group id for subject i */
    *ptra = alpha[j];
    *ptrb = beta[j];
  }

  free(alpha);
  free(beta);

} /* END: median_alpha_beta */


/* Function to obtain the permutation from ties  */
int getPerm_ties(Z, ngrp, ties, nties, all_perm, nclus, max_assign_clus)
int *Z, ngrp, *ties, nties, nclus, max_assign_clus;
int *all_perm; /* Permutation vector */
{
  int i, j, *ptr1, count, *perm, max_count;
  int nties2, ties2[nties], max_perm, index;

  max_count = -1;
  nties2    = 0;
  max_perm  = 0;

  /* Loop over each tied permutation */
  for (i=0; i<=nties; i++) {

    index = ties[i];
    perm  = &all_perm[index*nclus];

    /* Get the cluster count for this permutation */
    count = 0;
    for (j=0, ptr1=Z; j<ngrp; j++, ptr1++) {
      if (perm[*ptr1] == max_assign_clus) count++;
    }

    if (count > max_count) {
      max_perm  = index;
      max_count = count;
      nties2    = 0;
    } else if (count == max_count) {
      if (!nties2) {
        ties2[0] = max_perm;
        ties2[1] = index;
        nties2   = 1;
      } else {
        nties2++;
        ties2[nties2] = index;
      }
    }
  } /* END:  for (i=0; i<nties; i++) */

  if (!nties2) return(max_perm);

  /* If there are still ties, then randomly choose 1 */
  /*i = gsl_rng_uniform_int(rngptr, nties2+1);*/
  i = rand() % (nties2+1);

  max_perm = ties[i];

  return(max_perm);
}

/* Function to return the mean of alpha and beta*/
void new_clustering(M1, M2, everyN, assign, Nclus, Ngrp, z_file, alpha_file, beta_file, out_alpha, out_beta, out_z, all_perm, Nperm, Nties)
 int *M1, *M2, *everyN, *Nclus, *Ngrp, *assign, *Nperm, *Nties;
 int *all_perm; /* Matrix of permutations passed in by row */
 char **z_file, **alpha_file, **beta_file, **out_alpha, **out_beta, **out_z;
{
  int zint, i, j, k, eofFlag, *ptri, Z[*Ngrp], ngrp, ii, nclus, *ptri2, nperm, *perm;
  int max_i, count, max_count, nclusm1, obs, nties, ties[*Nclus], total_ties, max_assign_clus;
  int MM1, MM2, ngrpm1, everyNN, Nii;
  FILE *fptz, *fpta, *fptb, *fpt1, *fpt2, *fpt3;
  double alpha[*Nclus], beta[*Nclus], *ptrd;
  float x;
  /*
  const gsl_rng_type *T;
  gsl_rng *rngptr;
  gsl_rng_env_setup();
  T = gsl_rng_ranlux;
  rngptr = gsl_rng_alloc(T);
  gsl_rng_set(rngptr, 123456);
  */

  nclus   = *Nclus;
  ngrp    = *Ngrp;
  nperm   = *Nperm;
  nclusm1 = nclus - 1;
  MM1     = *M1;
  MM2     = *M2;
  ngrpm1  = ngrp - 1;
  everyNN = *everyN;

  /* Get the maximum cluster count for the assigned vec. Use vec ties to temporarily store the cluster counts */
  cluster_counts(ties, nclus, assign, ngrp);
  j = -1;
  for (i=0, ptri=ties; i<nclus; i++, ptri++) {
    if (*ptri > j) {
      max_assign_clus = i;
      j = *ptri;
    } 
  }

  eofFlag = 0;
  obs = 0;
  total_ties = 0;
  Nii = 0;

  /* Open the files */
  fptz = open_file(*z_file, "r");
  fpta = open_file(*alpha_file, "r");
  fptb = open_file(*beta_file, "r");
  fpt1 = open_file(*out_alpha, "w");
  fpt2 = open_file(*out_beta, "w");
  fpt3 = open_file(*out_z, "w");

  while(1) {
    obs++;
    if (feof(fptz)) break;
    if (obs > MM2) break; 

    /* Read in a row from the Z file */
    for (j=0, ptri=Z; j<ngrp; j++, ptri++) {
      ii = fscanf(fptz, "%d", &zint);
      if (ii == EOF) {
        eofFlag = 1;
        break;
      }
      *ptri = zint;
    }

    /* Read in a row from alpha and beta files */
    for (j=0, ptrd=alpha; j<nclus; j++, ptrd++) {
      ii = fscanf(fpta, "%g", &x);
      *ptrd = x;
    }
    for (j=0, ptrd=beta; j<nclus; j++, ptrd++) {
      ii = fscanf(fptb, "%g", &x);
      *ptrd = x;
    }

    if (eofFlag) break;
    if (obs < MM1) continue;

    /* Use every NN observations */
    Nii++;
    if (Nii == everyNN) {
      /* Reset Nii */
      Nii = 0;
    } else {
      /* Skip this observation */
      continue;
    }

    max_count = -1;
    max_i     = -1;
    nties     = 0;
   
    /* Loop over each permutation of clusters */
    for (i=0; i<nperm; i++) {
      /* ith row of all perm matrix */
      perm = &all_perm[i*nclus];
   
      count = 0;

      /* Compare the assign vector to a permutation of clusters in the z file */
      for (j=0, ptri=assign, ptri2=Z; j<ngrp; j++, ptri++, ptri2++) {
        if (*ptri == perm[*ptri2]) count++;
      } 
      if (count > max_count) {
        max_count = count;
        max_i     = i;
        nties     = 0;
      } else if (count == max_count) {
        if (!nties) {
          ties[0] = max_i;
          ties[1] = i;
          nties   = 1;
        } else {
          nties++;
          ties[nties] = i;
        }
      }
    }

    if (nties) {
      max_i = getPerm_ties(Z, ngrp, ties, nties, all_perm, nclus, max_assign_clus);
      total_ties++;
    }

    /* For the permutation that gives the max, reorder alpha and beta */
    perm = &all_perm[max_i*nclus];

    /* Write out alpha and beta */
    for (i=0, ptri=perm; i<nclusm1; i++, ptri++) {
      k = *ptri;
      fprintf(fpt1, "%g ", alpha[k]);
      fprintf(fpt2, "%g ", beta[k]);
    }
    k = *ptri;
    fprintf(fpt1, "%g\n", alpha[k]);
    fprintf(fpt2, "%g\n", beta[k]);
    ii = fflush(fpt1);  
    ii = fflush(fpt2);

    /* Write out z */
    for (i=0, ptri=Z; i<ngrpm1; i++, ptri++) fprintf(fpt3, "%d ", perm[*ptri]);
    fprintf(fpt3, "%d\n", perm[*ptri]);
    ii = fflush(fpt3);  

  } /* END: while */  

  fclose(fptz);
  fclose(fpta);
  fclose(fptb);
  fclose(fpt1);
  fclose(fpt2);
  fclose(fpt3);
  /*gsl_rng_free(rngptr);*/

  *Nties = total_ties;

} /* END: new_clustering */


